@@ -33,9 +33,10 @@ class TestDataLoader(unittest.TestCase):
         tokenizer = BertTokenizer.from_pretrained('bert-base-cased')
         for i in tfkit.gen_onebyone.get_data_from_file('../demo_data/generate.csv'):
             print(i)
+        maxlen = 512
         for likelihood in ['onebyone-neg', 'onebyone-pos', 'onebyone-both']:
             for i in tfkit.gen_onebyone.loadOneByOneDataset('../demo_data/generate.csv', pretrained='bert-base-cased',
-                                                            maxlen=24, likelihood=likelihood):
+                                                            maxlen=maxlen, likelihood=likelihood):
                 print(likelihood, i)
                 start_pos = i['start']
                 self.assertTrue(tokenizer.mask_token_id == i['input'][start_pos])
@@ -43,8 +44,8 @@ class TestDataLoader(unittest.TestCase):
                     self.assertTrue(i['ntarget'][start_pos] != -1)
                 else:
                     self.assertTrue(i['target'][start_pos] != -1)
-                self.assertTrue(len(i['input']) == 24)
-                self.assertTrue(len(i['target']) == 24)
+                self.assertTrue(len(i['input']) == maxlen)
+                self.assertTrue(len(i['target']) == maxlen)
 
     def testClassifier(self):
         for i in tfkit.classifier.get_data_from_file('../demo_data/classification.csv'):
